@@ -702,6 +702,9 @@ class Observer(SchedulerBase):
 
         # print(moon_pos)
 
+        if malt < -5:
+            return np.zeros(len(targ_ra))
+
         deltaV = KS91_deltaV(alpha, moon_targ_dist, zee, zee_m)
 
         # if len(moon_targ_dist) > 1:
@@ -1030,9 +1033,15 @@ class Scheduler(Master):
         moon_dist = self.moon_dist(mjd=mjd, ra=self.fields.racen,
                                    dec=self.fields.deccen)
 
-        # whereRM = np.where(["bhm_rm" in c for c in self.fields.cadence])[0]
+        # skybrightness in 2 days. Checked at each slot. Gives two chances to
+        # finish partial field if scheduled
+        skybrightness_2days = self.skybrightness(mjd+2)
+
+        whereRM = np.where(["174x" in c for c in self.fields.cadence])[0]
         # where_uhoh = np.where(["dark_2x" in c for c in self.fields.cadence])[0]
 
+        # print("\n")
+
         next_change, next_brightness = self.next_change(mjd)
 
         nexp_change = int((next_change - mjd) / self.exp_time)
@@ -1042,10 +1051,11 @@ class Scheduler(Master):
                 # it's still dark
                 nexp_change = 1e3
             elif nexp_change == 0:
-                # going into dark from bright, don't waste 15 minutes?
-                # delta V check will cover bad cases
-                skybrightness = 0.34
-                nexp_change = maxExp
+                # going into dark from bright, don't waste time
+                # delta V check might cover bad cases, don't chance it
+                # let bright encroach to dark
+                # skybrightness = 0.34
+                nexp_change = 1
         else:
             # next is bright
             if skybrightness > 0.35:
@@ -1058,8 +1068,6 @@ class Scheduler(Master):
 
         # print(f"{float(mjd):.3f} {float(next_change):.3f} {float(next_brightness):.2f} {nexp_change}", maxExp)
 
-        # over = list()
-
         # indxs = np.where(self.fields.nextmjd > mjd)[0]
         # observable[indxs] = False
         indxs = np.where(observable)[0]
@@ -1110,9 +1118,24 @@ class Scheduler(Master):
             epoch_idxs[indx] = epoch_idx
 
             if nexp[indx] > nexp_change and self.fields.flag[indx] != 1:
-                # change between bright/dark, field doesn't fit
-                observable[indx] = False
-                continue
+                if cadence.max_length[epoch_idx] < 1:
+                    # not enough time for one-night epoch
+                    observable[indx] = False
+                    continue
+
+            if nexp[indx] > 4 or airmass[indx] > 1.3:
+                endmjd = mjd + nexp[indx] * self.exp_time
+                (alt, az) = self.radec2altaz(mjd=endmjd,
+                                             ra=self.fields.racen[indx],
+                                             dec=self.fields.deccen[indx])
+                endam = self.alt2airmass(alt)
+                if endam > airmass[indx]:
+                    airmass[indx] = endam
+
+            verbose = False
+            # if cadence.nexp[epoch_idx] == 2:
+            #     print(int(self.fields.pk[indx]), observable[indx], f"{airmass[indx]:3.1f}", alt, cadence.nexp[epoch_idx], cadence.label_root)
+            #     verbose = True
 
             observable[indx], delta_priority[indx] =\
                 cadence.evaluate_next(epoch_idx=epoch_idx,
@@ -1123,7 +1146,7 @@ class Scheduler(Master):
                                       moon_dist=moon_dist[indx],
                                       deltaV=deltav[indx],
                                       airmass=airmass[indx],
-                                      verbose=False)
+                                      verbose=verbose)
 
             percent_done = len(mjd_past) / expCount[-1]
 
@@ -1138,9 +1161,13 @@ class Scheduler(Master):
             # prioritize nepochs. Huge bump for 8 + epochs
             # slight decrement for single epochs
             delta_priority[indx] += (cadence.nepochs - 2)*self.nepochsPri
-            if nexp[indx] > maxExp:
-                # print(f"{float(mjd):.3f} {int(self.fields.pk[indx])} c_nexp {cadence.nexp[epoch_idx]} MAXEXP {maxExp}")
-                observable[indx] = False
+            if nexp[indx] > maxExp and observable[indx]:
+                if cadence.max_length[epoch_idx] < 1\
+                   or skybrightness_2days > 0.35:
+                    # can't schedule partial epoch
+                    observable[indx] = False
+                else:
+                    nexp[indx] = maxExp
             if self.fields.flag[indx] == 1:
                 # flagged as top priority
                 # if we're in this loop, it's above the horizon
@@ -1149,6 +1176,9 @@ class Scheduler(Master):
                 observable[indx] = True
                 delta_priority[indx] += 1e6
 
+            # if cadence.nexp[epoch_idx] > 4:
+            #     print(int(self.fields.pk[indx]), observable[indx], delta_priority[indx], cadence.name)
+
         iobservable = np.where(observable)[0]
 
         return iobservable, nexp[iobservable], delta_priority[iobservable],\
